/*
 * Copyright 1993-2009 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and 
 * proprietary rights in and to this software and related documentation. 
 * Any use, reproduction, disclosure, or distribution of this software 
 * and related documentation without an express license agreement from
 * NVIDIA Corporation is strictly prohibited.
 *
 * Please refer to the applicable NVIDIA end user license agreement (EULA) 
 * associated with this source code for terms and conditions that govern 
 * your use of this NVIDIA software.
 * 
 */

/* Template project which demonstrates the basics on how to setup a project 
* example application.
* Host code.
*/

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include <cutil_inline.h>

// includes, kernels
#include <template_kernel.cu>

////////////////////////////////////////////////////////////////////////////////
// declaration, forward
void runTest( int argc, char** argv);

extern "C"
void computeGold( float d, float* reference, int imageW, int imageH, float4* fd);

////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////
int
main( int argc, char** argv) 
{
	runTest( argc, argv);
	cutilExit(argc, argv);
}

////////////////////////////////////////////////////////////////////////////////
//! Run a simple test for CUDA
////////////////////////////////////////////////////////////////////////////////
void
runTest( int argc, char** argv) 
{
	// use command-line specified CUDA device, otherwise use device with highest Gflops/s
	if( cutCheckCmdLineFlag(argc, (const char**)argv, "device") )
		cutilDeviceInit(argc, argv);
	else
		cudaSetDevice( cutGetMaxGflopsDeviceId() );

    	unsigned int num_threads = 32;
    	unsigned int mem_size = sizeof( float) * num_threads;

	// allocate device memory for result
	float* d_odata;
	cutilSafeCall( cudaMalloc( (void**) &d_odata, mem_size));

	// allocate mem for the result on host side
	float* h_odata = (float*) malloc( mem_size);
	
	// setup execution parameters
	dim3  grid( 1, 1, 1);
	dim3  threads( num_threads, 1, 1);

	//SETUP CONSTANT MEMORY
	float4 tmpd[4];
	int* tmpdi = (int*)&(tmpd[0].x);

	tmpdi[ 0] = 0x424d49f5;
	tmpdi[ 1] = 0x40c9d0ff;
	tmpdi[ 2] = 0x3f208ee8;
	tmpdi[ 3] = 0x3e134f49;
	tmpdi[ 4] = 0x424a97e3;
	tmpdi[ 5] = 0x40e91b65;
	tmpdi[ 6] = 0x3f1f1d80;
	tmpdi[ 7] = 0x3e134f49;
	tmpdi[ 8] = 0x424e0a39;
	tmpdi[ 9] = 0x40cdd1c5;
	tmpdi[10] = 0xbe6ecfdc;
	tmpdi[11] = 0x3e134f49;
	tmpdi[12] = 0xc24ca2ff;
	tmpdi[13] = 0xc0e93cea;
	tmpdi[14] = 0xbeeb9dc7;
	tmpdi[15] = 0xbe114300;

	cudaMemcpyToSymbol ("fd", tmpd, 4*sizeof(float4), 0, cudaMemcpyHostToDevice );

	float* reference = (float*) malloc( mem_size);
		
	int 	range 	= 10;
	float 	step 	= 1.0f/range;

	for (int k=-range;k<=range;++k)
	{
		float d = k*step;

		// execute the kernel
		testKernel<<< grid, threads, mem_size >>>( d, d_odata, num_threads, 1);

		// check if kernel execution generated and error
		cutilCheckMsg("Kernel execution failed");

		// copy result from device to host
		cutilSafeCall( cudaMemcpy( h_odata, d_odata, sizeof( float) * num_threads, cudaMemcpyDeviceToHost) );
	
		//fprintf(stderr,"d = %f\n",d);
		//fprintf(stderr,"test = %f\n",h_odata[0]);

		// compute reference solution
		computeGold( d, reference, num_threads, 1, tmpd);

		fprintf(stderr,"d = %f\n",d);

		bool passed = true;		
		for (int i=0;i<num_threads;++i)
			passed = passed && (fabs(h_odata[i]-reference[i])<0.0001f);
		printf( "Test %s\n", passed ? "PASSED" : "FAILED");

	}


	// cleanup memory
	free( h_odata);
	free( reference);
	cutilSafeCall(cudaFree(d_odata));

	cudaThreadExit();
}
